/********************************************************/
/*  Profile  Program                                    */
/*   This program demonstrates some of the classes      */
/*   for maintaining OS/2 mle and files                 */
/********************************************************/

#include <iapp.hpp>
#include <istattxt.hpp>
#include <imenubar.hpp>
#include <isubmenu.hpp>
#include <ititle.hpp>
#include <ipopmenu.hpp>
#include <ifont.hpp>
#include <ifiledlg.hpp>
#include <istring.hpp>
#include <ibuffer.hpp>
#include <ilistbox.hpp>
#include <isetcv.hpp>
#include <isplitcv.hpp>
#include <ipushbut.hpp>
#include <iprofile.hpp>
#include <imle.hpp>
#include <ifont.hpp>
#include <ifontdlg.hpp>
#include <imsgbox.hpp>
#include <iclipbrd.hpp>
#include <itbar.hpp>
#include <itbarbut.hpp>
#include <iflytext.hpp>
#include <ithread.hpp>
#include <ievent.hpp>
#include <stdlib.h>
//define INCL_WINHELP
#define INCL_PM
#include <os2/os2.h>

#include "mainwnd.hpp"
#include "colordlg.hpp"
#include "mainwnd.h"

/**********************************************************/
/*   Global data definitions                             **/
/**********************************************************/

char  *pCh1, *pCh2=NULL;

/**********************************************************/
/*   Main application entry point                        **/
/**********************************************************/

void main(int nArg, char* cArg[])
{
 pCh1 = cArg[0];                    /* set the pointers to */
 if (nArg > 1)                      /* args if present     */
   pCh2 = cArg[1];

 AProfWindow mainWindow (ID_MAINWND);
 IApplication::current() .run();
}

/*********************************************************/
/*  Constructor for main window                         **/
/*-------------------------------------------------------*/
/* (1) construct main window and associated controls     */
/* (2) establish command and menu   message handlers     */
/* (3) control and class data initialization             */
/* (4) select and show the window and client controls    */
/*********************************************************/

AProfWindow :: AProfWindow(unsigned long windowId)
   : IFrameWindow (
     IFrameWindow::defaultStyle()
     | IFrameWindow::minimizedIcon
     | IFrameWindow::accelerator,
     windowId)
  ,titleBar(this, IResourceId(ID_MAINWND) )
  ,menuBar(windowId, this)
  ,fileTools(IDTB_FILE, this)
  ,editTools(IDTB_EDIT, &fileTools, true)
  ,openButton(IC_ID_OPEN,    &fileTools, &fileTools)
  ,saveButton(IC_ID_SAVE,    &fileTools, &fileTools)
  ,printButton(IC_ID_PRINT,   &fileTools, &fileTools)
  ,cutButton(IC_ID_CUT, &editTools, &editTools)
  ,copyButton(IC_ID_COPY, &editTools, &editTools)
  ,pasteButton(IC_ID_PASTE, &editTools, &editTools)
  ,isCnv1(ID_CANV1, this, this)
  ,mle1(IDW_MLE, &isCnv1, &isCnv1, IRectangle(),
           IMultiLineEdit::defaultStyle() )
  ,mle2(IDW_MLE2, &isCnv1, &isCnv1, IRectangle(),
           IMultiLineEdit::defaultStyle() )
  ,flyText( STR_FLY_OPEN, &menuBar )
  ,flyOver(&flyText, 1000, 250)
  ,icClear("clear")
  ,icQuit("quit")
  ,icTop("top")
  ,icBottom("bottom")
  ,icFind("find")
  ,icChange("change")
  ,icUp("up")
  ,icDown("down")
{
  ICommandHandler::handleEventsFor(this);
  IMenuHandler::handleEventsFor(this);
  ICommandHandler::handleEventsFor(&mle1);
  ICommandHandler::handleEventsFor(&mle2);
  IKeyboardHandler::handleEventsFor(&mle2);

  popUpHandler.handleEventsFor(&mle1);
  popUpHandler.handleEventsFor(&mle2);

  flyOver.setHelpText(openButton.handle(), IResourceId(STR_FLY_OPEN),
                                           IResourceId(0) );
  flyOver.setHelpText(saveButton.handle(), IResourceId(STR_FLY_SAVE),
                                           IResourceId(0) );
  flyOver.setHelpText(printButton.handle(), IResourceId(STR_FLY_PRINT),
                                           IResourceId(0) );
  flyOver.setHelpText(cutButton.handle(), IResourceId(STR_FLY_CUT),
                                           IResourceId(0) );
  flyOver.setHelpText(copyButton.handle(), IResourceId(STR_FLY_COPY),
                                           IResourceId(0) );
  flyOver.setHelpText(pasteButton.handle(), IResourceId(STR_FLY_PASTE),
                                           IResourceId(0) );

  flyOver.handleEventsFor(&fileTools);
  flyOver.handleEventsFor(&editTools);

  isCnv1.setOrientation( ISplitCanvas::horizontalSplit );
  setClient(&isCnv1);

  isCnv1.setSplitWindowPercentage(&mle1, 80);
  isCnv1.setSplitWindowPercentage(&mle2, 20);

  strcpy(prfName, "*.*");
  bPrFlag = FALSE;
  bBrowse = FALSE;
  bPrtAvail = TRUE;

  mle1.setFont(IFont("System VIO", 12));          //default font
  mle1.setBackgroundColor(IColor(255, 255, 255)); //and color
  mle1.resetChangedFlag();
  mle1.disableWordWrap();
  mle2.setBackgroundColor(IColor(128, 128, 200)); //background
  mle2.setForegroundColor(IColor(255, 255, 000)); //text

  titleBar.setViewText(IResourceId(STR_UNNAMED)); //no file

  fileTools.addAsLast(&openButton, true);
  fileTools.addAsLast(&saveButton);
  fileTools.addAsLast(&printButton);

  editTools.addAsLast(&cutButton, true);
  editTools.addAsLast(&copyButton);
  editTools.addAsLast(&pasteButton);

  ALoadCmds();

  if (pCh2)                 //load commandline file
    ALoadFile(pCh2);        //if specified
  else
  {
    ulCline = 0;
    ulPos   = 0;
    ulLPos  = 0;
    ulMaxLine = 0;
    ulFLine = 0;
    ulFPos  = 0;
  }

  setFocus();
  show();
}

/*********************************************************/
/*  Command Handler for main window                     **/
/*-------------------------------------------------------*/
/* Handles WM_COMMAND messages for main window and its   */
/*   controls                                            */
/*********************************************************/

Boolean AProfWindow :: command(ICommandEvent & cmdEvent)
{
 unsigned short value;
 unsigned long  ulMid;

 switch (cmdEvent.commandId())
 {
//
//  Menu Item commands
//
   case IDC1_OPEN:    /* select a new file, using file dialog */
   case IC_ID_OPEN:
   {
    int x, y=strlen(prfName);

    if (bPrFlag)                   /* file already present */
      return(true);                /*  ignore the open     */

    for (x=y-1; x>0; x--)
      if (prfName[x] == '\\')
      {
        prfName[x+1]=0;            /* terminate string */
        strcat(prfName, "*.*");    /* concatenate wildcard */
        x=0;                       /* end loop             */
      }

    IFileDialog::Settings fileSettings;
    fileSettings.setTitle("Open file");
    fileSettings.setFileName(prfName);
    IFileDialog fileDialog(this, this, fileSettings);
    if ( fileDialog.pressedOK() )     /* process if OKed */
    {
      ALoadFile(fileDialog.fileName() );
    }
    return(true);
   }

   case IDC1_CLOSE:
   {
     if (!bBrowse)                    //if edit mode
       if (!ACheckSave(TRUE) )          //check save
         return(true);                  //if CANCEL specified

     if (bPrFlag)                     //if there is a file
     {
       bPrFlag = FALSE;               //  reset the flag
       mle1.removeAll();              //  empty the contents
       mle1.resetChangedFlag();
       prfName[0]=0;                  //  clear the file name
       titleBar.setViewText(IResourceId(STR_UNNAMED)); //no file
     }
     return(true);
   }

   case IDC1_SAVEAS:  /* select a new profile, using file dialog */
   {
    ASaveAsFile();
    return(true);
   }

   case IDC1_SAVE:
   case IC_ID_SAVE:
   {
     if (bPrFlag)                     //if there is a file
     {
       ASaveFile();
     }
     return(true);
   }

   case IDC1_PRINT:
   case IC_ID_PRINT:
   {
     if (mle1.textLength() && bPrtAvail )
     {
      APrintFn *prnFn = new APrintFn( *this, UWM_THREADEND );
      IThread thread(prnFn);
     }

     return(true);
   }

   case IDE_UNDO:
   {
     mle1.undo();
     return(true);
   }

   case IDE_CLEAR:
   {
     mle1.removeAll();                //  empty the contents
     mle1.resetChangedFlag();
     return(true);
   }

   case IDE_DELETE:             //delete
   {
     if (mle1.hasSelectedText() )
     {
       mle1.discard();
       mle1.setChangedFlag();
     }
     return(true);
   }

   case IC_ID_CUT:              //toolbar CUT, only if edit mode

     if (bBrowse)
       return(true);

   case IDE_CUT:                //cut to clipoard
   {
     if (mle1.hasSelectedText() )
     {
       mle1.cut();
       mle1.setChangedFlag();
     }
     return(true);
   }

   case IC_ID_COPY:             //toolbar copy
   case IDE_COPY:               //copy to clipboard
   {
     if (mle1.hasSelectedText() )
     {
       mle1.copy();
     }
     return(true);
   }

   case IC_ID_PASTE:

     if ( (bBrowse) || (!mle1.clipboardHasTextFormat()) )
       return(true);

   case IDE_PASTE:              //paste from clipboard
   {
     mle1.paste();
     mle1.setChangedFlag();
     return(true);
   }

   case IDO_WWRAP:              //toggle word wrap
   {
     mle1.enableWordWrap(!menuBar.isItemChecked(IDO_WWRAP));
     return(true);
   }

   case IDO_BROWSE:             //toggle browse mode
   {
     bBrowse = !bBrowse;                   //toggle the flag
     mle1.enableDataUpdate(!bBrowse);      //and the update mode
     return(true);
   }

//   Popup messages
   case IDP_FONT:
   {
    popupOwner = cmdEvent.window();
    IFont cFont(popupOwner);
    IFontDialog::Settings fontSettings(&cFont);
    fontSettings.setTitle("Change Font");
    IFontDialog fontDialog( desktopWindow(),
                            this,
                            IFontDialog::resetButton,
                            fontSettings);
    if (fontDialog.pressedOK() )
    {
     popupOwner->setFont(cFont);
    }
    return(true);
   }

   case IDP_COLORB:
   {
    ULONG ulRc;

    popupOwner = cmdEvent.window();
    clrDialog = new AClrDialog( "Background", popupOwner );
    ulRc = clrDialog->showModally();
    delete clrDialog;
    return(true);
   }

   case IDP_COLORF:
   {
    ULONG ulRc;

    popupOwner = cmdEvent.window();
    clrDialog = new AClrDialog( "Foreground", popupOwner );
    ulRc = clrDialog->showModally();
    delete clrDialog;
    return(true);
   }

//   Other Key Commands
   case IDK_CMDWND:
   {
     mle2.setFocus();                  //switch to command window
     return(true);
   }

//   Exit messages
   case IDC1_EXIT:
   case ID_EXITPROGRAM:
   {
     if (!bBrowse)                     //if edit mode
       if (!ACheckSave(TRUE) )         //  check for save needed
          return(true);

     close();
     return(true);
   }

 }

 return(false);      /* all others, indicate not processed */
}

/*********************************************************/
/*  System Command Handler for Main Window              **/
/*-------------------------------------------------------*/
/* Handles WM_SYSCOMMAND messages for main window        */
/*********************************************************/

Boolean AProfWindow :: systemCommand(ICommandEvent & sysCmdEvent)
{
 switch (sysCmdEvent.commandId())
 {
//
   case SC_CLOSE:
   {
     if (!bBrowse)                     //if edit mode
       if (!ACheckSave(TRUE) )         //  check for save needed
          return(true);

     close();
     return(true);
   }

    default:
      return(false);

 }

 return(false);      /* all others, indicate not processed */
}

/*********************************************************/
/*  Menu Handler for main window                        **/
/*-------------------------------------------------------*/
/* Handles menu activation events                        */
/*********************************************************/

Boolean AProfWindow :: menuShowing(IMenuEvent & mnEvent,
                                   ISubmenu   & smnAboutToShow)
{
  BOOL bCond = mle1.hasSelectedText();

  switch( smnAboutToShow.id() )
  {
    case IDC_FILE:                    //FILE submenu
    {
      AEnableFileMenu(bPrFlag);
      return(true);
    }

    case IDE_EDIT:                    //enable/disable menu items
    {
      if (bBrowse)                    //separate disables if browse
      {
        menuBar.enableItem(IDE_UNDO,  false);       //allow only
        menuBar.enableItem(IDE_CLEAR, false);       //
        menuBar.enableItem(IDE_DELETE, false);      //
        menuBar.enableItem(IDE_CUT, false);         //
        menuBar.enableItem(IDE_COPY, bCond);        //copy (if selection)
        menuBar.enableItem(IDE_PASTE, false);       //
      }
      else
      {
        menuBar.enableItem(IDE_UNDO, mle1.isUndoable() );
        menuBar.enableItem(IDE_CLEAR, true);
        menuBar.enableItem(IDE_DELETE, bCond );
        menuBar.enableItem(IDE_CUT,  bCond );
        menuBar.enableItem(IDE_COPY, bCond );
        menuBar.enableItem(IDE_PASTE, mle1.clipboardHasTextFormat() );
      }

      return(true);
    }

    case IDO_OPTIONS:                 //OPTIONS submenu
    {
      menuBar.checkItem( IDO_WWRAP, mle1.isWordWrap() );
      menuBar.checkItem( IDO_BROWSE, bBrowse );
      return(true);
    }

    default:
      return(false);
  }

  return(false);

}

/*********************************************************/
/*  Keyboard Handler for MLE2                           **/
/*-------------------------------------------------------*/
/* Handles virtualkey press events                       */
/*********************************************************/

Boolean AProfWindow :: virtualKeyPress(IKeyboardEvent& ikyEvent)
{
  switch( ikyEvent.virtualKey() )
  {
    case IKeyboardEvent::newLine:     //newline
    {
      IString cString(mle2.text(mle2.cursorLinePosition()) );

      AExecuteCmd( cString );
      return(false);
    }

    default:
      return(false);
  }

  return(false);

}

IBase :: Boolean APopUpHandler::makePopUpMenu(IMenuEvent &menuEvent)
{
  IPopUpMenu *popUpMenu;
  popUpOwner = menuEvent.window();

  popUpMenu = new IPopUpMenu(IDP_POPUP, popUpOwner);
  if (popUpMenu)
  {
    popUpMenu->setAutoDeleteObject();
    popUpMenu->show(menuEvent.mousePosition() );
  }
  return(true);
} /* end APopUpHandler */

/*********************************************************/
/*  Load specified file into mle1                       **/
/*-------------------------------------------------------*/
/*********************************************************/

void AProfWindow :: ALoadFile(char *fName)
{
   if (bPrFlag)
   {
     bPrFlag=FALSE;             /* clean out prev profile */
   }
   strcpy(prfName, fName);            //save the new name
   titleBar.setViewText(prfName);     //reset the view text
   mle1.removeAll();                  //clear the control
   mle1.importFromFile(prfName);      //load the file
   mle1.setTop(0);                    //reset to top of file
   mle1.setCursorLinePosition(0);
   mle1.resetChangedFlag();           //indicate unchanged
   bPrFlag = TRUE;                    //indicate file is present

   ulCline = 0;
   ulMaxLine = mle1.numberOfLines();
   ulPos = 0;
   ulLPos = 0;
   ulFLine = 0;
   ulFPos = 0;

}

/*********************************************************/
/*  Enable/Disable FILE menu items                      **/
/*-------------------------------------------------------*/
/*********************************************************/

void AProfWindow :: AEnableFileMenu(BOOL bFlag)
{
   if (bBrowse)                      // if browse-only --->
   {
     menuBar.enableItem(IDC1_OPEN, !bFlag);     //allow open if no file
     menuBar.enableItem(IDC1_CLOSE, bFlag);     //allow close if file
     menuBar.enableItem(IDC1_SAVE,  false);     //never allow save
     menuBar.enableItem(IDC1_SAVEAS, false);    //never allow saveas
     menuBar.enableItem(IDC1_PRINT, mle1.textLength() && bPrtAvail );
   }
   else
   {                                 // if edit mode ---->
     BOOL bData = mle1.isChanged();

     menuBar.enableItem(IDC1_OPEN, !bFlag);     //allow open if no file
     menuBar.enableItem(IDC1_CLOSE, bFlag);     //allow close if file
     menuBar.enableItem(IDC1_SAVE,  bFlag && bData);   //allow save  if changed
     menuBar.enableItem(IDC1_SAVEAS, mle1.textLength() );  //allow if data
     menuBar.enableItem(IDC1_PRINT, mle1.textLength() && bPrtAvail );
   }
}

/*********************************************************/
/*  Check for Save                                      **/
/*-------------------------------------------------------*/
/*********************************************************/

Boolean AProfWindow :: ACheckSave(BOOL bFlag)
{
   if (mle1.isChanged() )           //if the data content has changed
   {
    IMessageBox saveBox( desktopWindow() );       //show message box
    saveBox.setTitle(IResourceId(STR_TITLE1));    // title from RC

    switch (saveBox.show(IResourceId(STR_CHANGED),
                     IMessageBox::yesNoCancelButton |
                     IMessageBox::moveable |
                     IMessageBox::warningIcon) )
    {
      case IMessageBox::cancel:                 //cancel,
           return(false);                       //  indicate bypass

      case IMessageBox::no:                     //no,
           return(true);                        //  continue process

      case IMessageBox::yes:                    //yes,
      {
           if (bPrFlag)                         //  save to file if
             ASaveFile();                       //   there is a file
           else
             ASaveAsFile();                     //  otherwise, prompt
           return(true);
      }
    }

   }
   return(true);
}

/*********************************************************/
/*  Save to File                                        **/
/*-------------------------------------------------------*/
/*********************************************************/

void AProfWindow :: ASaveFile()
{
  if ( mle1.isChanged() )
  {
    mle1.exportToFile(prfName);    //  export it
    mle1.resetChangedFlag();
  }
}

/*********************************************************/
/*  Save to File with prompt                            **/
/*-------------------------------------------------------*/
/*********************************************************/

void AProfWindow :: ASaveAsFile()
{
    if ( mle1.isChanged() )
    {
      IFileDialog::Settings fileSettings;
      fileSettings.setSaveAsDialog();
      fileSettings.setFileName(prfName);
      IFileDialog fileDialog(this, this, fileSettings);
      if ( fileDialog.pressedOK() )     /* process if OKed */
      {
        strcpy(prfName, fileDialog.fileName() );
        bPrFlag = TRUE;
        titleBar.setViewText(prfName);
        mle1.exportToFile(prfName);
        mle1.resetChangedFlag();
      }
    }
}

/*********************************************************/
/*  Load Command Strings                                **/
/*-------------------------------------------------------*/
/*********************************************************/

void AProfWindow :: ALoadCmds( )
{
  return;   //dummy for now, until I figure out how this works
}           // (strings are loaded at initialize time)

/*********************************************************/
/*  Process EDIT command                                **/
/*-------------------------------------------------------*/
/*********************************************************/

void AProfWindow :: AExecuteCmd(IString& isCmdLine)
{
  unsigned nWords = isCmdLine.numWords();      //number of words
  IString isCmd = isCmdLine.word( 1);          //command

  if (  isCmd ==  icClear )
  {
    mle2.removeAll();
    return;
  }

  if (  isCmd ==  icTop   )
  {
    mle1.setTop(0);
    return;
  }

  if (  isCmd ==  icBottom )
  {
    unsigned long ulLines = mle1.numberOfLines(),
                  ulVis   = mle1.visibleLines();
    if ( ulLines <= ulVis )
       mle1.setTop(0);
    else
       mle1.setTop( ulLines - ulVis );

    return;
  }

  if (  isCmd ==  icDown   )
  {
    unsigned long ulLines = mle1.numberOfLines(), //total lines
                  ulVis   = mle1.visibleLines(),  //display size
                  cTop    = mle1.top(),           //current top
                  cLines  = 1;

    if ( isCmdLine.numWords() > 1 )
    {
      cLines = isCmdLine.word( 2 ).asUnsigned();
    }
    if ( (cTop + cLines) <= ulLines )
       mle1.setTop(cTop+cLines);
    else
       mle1.setTop( ulLines - ulVis );

    return;
  }

  if (  isCmd ==  icUp     )
  {
    unsigned long ulLines = mle1.numberOfLines(), //total lines
                  ulVis   = mle1.visibleLines(),  //display size
                  cTop    = mle1.top(),           //current top
                  cLines  = 1;

    if ( isCmdLine.numWords() > 1 )
    {
      cLines = isCmdLine.word( 2 ).asUnsigned();
    }
    if ( cTop >= cLines )
       mle1.setTop(cTop - cLines);
    else
       mle1.setTop( 0 );

    return;
  }

  if ( isCmd == icQuit )
  {
    if (!bBrowse)                     //if edit mode
      if (!ACheckSave(TRUE) )         //  check for save needed
         return;

    close();
    return;
  }

}

/*********************************************************/
/*  Print on Thread Function                            **/
/*-------------------------------------------------------*/
/*  this function will be called from a separate thread  */
/*********************************************************/

AProfWindow& AProfWindow :: printOnThread(unsigned long eventId)
{
     if (mle1.textLength() )
     {
       bPrtAvail = FALSE;                            //printer busy
       IMessageBox saveBox( desktopWindow() );       //show message box
       saveBox.setTitle("Message from Print Thread");

       saveBox.show("Print not yet supported",
                    IMessageBox::okButton |
                    IMessageBox::moveable |
                    IMessageBox::informationIcon);
       bPrtAvail = TRUE;                             //printer available
     }
     postEvent( eventId );
     return *this;
}
