/************************************************************/
/*---------Class definitions for Profile  program-----------*/
/************************************************************/

#include <icmdhdr.hpp>         //ICommand handler
#include <iselhdr.hpp>         //ISelect handler
#include <imenuhdr.hpp>        //IMenuHandler handler
#include <ikeyhdr.hpp>         //IKeyboardHandler handler
#include <iflyhhdr.hpp>        //IFlyOverHelpHandler handler
#include <os2/os2.h>

class    IPushButton;
class    IListBox;
class    IString;
class    IFrameWindow;
class    IProfile;
class    IMultiLineEdit;
class    ISetCanvas;
class    ISplitCanvas;
class    ITitle;
class    IToolBar;
class    IToolBarButton;
class    AClrDialog;
class    IFlyOverHelpHandler;
class    IFlyText;
class    IThread;

/***********************************************************/
/*  Popup Handler class definition                        **/
/***********************************************************/

class APopUpHandler : public IMenuHandler
{
    protected:
      virtual Boolean makePopUpMenu(IMenuEvent& menuEvent);

    public:
      IWindow    *popUpOwner;
};

/***********************************************************/
/*  Main Class definition                                 **/
/***********************************************************/

class AProfWindow : public IFrameWindow,
                    public IMenuHandler,
                    public IKeyboardHandler,
                    public ICommandHandler
{
  public:
    AProfWindow(unsigned long windowId);   //constructor
    void ALoadFile(char *fName);           //load from file
    void ASaveFile();                      //save to file
    void ASaveAsFile();                    //save with prompt
    void AEnableFileMenu(BOOL bFlag);      //set menu items
    Boolean ACheckSave(BOOL bFlag);        //check for save
    void ALoadCmds();                      //load command table
    void AExecuteCmd(IString &isCmd);      //edit command
    AProfWindow &printOnThread(unsigned long eventId); //print function

  protected:
    Boolean command(ICommandEvent& cmdEvent);    //command messages
    Boolean systemCommand(ICommandEvent& sysCmdEvent); //sys command messages
    Boolean menuShowing(IMenuEvent& mnEvent,
                        ISubmenu& smnAboutToShow);
    Boolean virtualKeyPress(IKeyboardEvent& ikyEvent);

  private:
    ITitle           titleBar;       //main window title
    IMenuBar         menuBar;        //main window menu
    IToolBar         fileTools,      //FILE tool bar
                     editTools;
    IToolBarButton   openButton,
                     saveButton,
                     printButton,
                     cutButton,
                     copyButton,
                     pasteButton;
    ISplitCanvas     isCnv1;
    IMultiLineEdit   mle1, mle2;     //value multiline edit
    IProfile        *myProfile;      //profile object
    IProfile::Cursor *myCursor;      //profile appname cursor

    IWindow         *popupOwner, *keyOwner;
    APopUpHandler    popUpHandler;
    AClrDialog      *clrDialog;

    IFlyOverHelpHandler  flyOver;
    IFlyText             flyText;

    IThread           pThread;       //print thread

    IString           icClear,       //command strings
                      icTop,
                      icBottom,
                      icFind,
                      icChange,
                      icUp,
                      icDown,
                      icQuit;

    char             prfName[64];    //current profile name
    BOOL             bPrFlag;        //profile active flag
    BOOL             bBrowse;        //browse only flag
    BOOL             bPrtAvail;      //printer thread available
    char             cvMode;         //hex or char mode
    unsigned long    ulCline,        //current line number
                     ulMaxLine,      //max line number
                     ulPos,          //current file position
                     ulLPos,         //current line position
                     ulFLine,        //last fine line
                     ulFPos;         //last find position
 };

/***********************************************************/
/*  Thread function for printing                          **/
/***********************************************************/

class APrintFn : public IThreadFn
{
   public:
     APrintFn(AProfWindow &obj, unsigned long evtId)
      : object(obj)
      , eventId(evtId)
        {;}
     void run() { object.printOnThread(eventId); }

   private:
     AProfWindow    &object;
     unsigned long  eventId;
};
